/*
 * sound.v
 *
 * Circuit to generate PDM sound using an Altera Nios board with an
 * AleaRep Lancelot sound add-on.
 *
 * This module should be fed with a 200 MHz clock from the PLL.
 */

module sound(
	clk,
	data_l,
	data_r,
	audio_l,
	audio_r,
	audio_clk
	);

   input	clk;
   input [11:0] data_l;
   input [11:0] data_r;
   output	audio_l;
   output	audio_r;
   output       audio_clk;

   reg [12:0] 	pulse_ctr;
   reg [12:0] 	gen_ctr_l;
   reg [12:0] 	gen_ctr_r;
 
   // This gives 12-bit sound at 44100 Hz.  A high sample rate will
   // help avoid actually making the sampling frequency audible.
   // The low-pass filter on the Lancelot has its knee at 10 kHz, and
   // (being a first-order filter) drops off at 3 dB/octave.

   // Note: the input is expected unsigned.  For signed input, invert
   // bit 11.
  
   always @(posedge clk)
     begin
	if ( pulse_ctr == 0 )
	  begin
	     pulse_ctr <= 4535-1;      // 44100 Hz sample rate
	     gen_ctr_l <= data_l+256;  // Really should be 218, but 256 is
	     gen_ctr_r <= data_r+256;  // close enough and cheaper to add
	  end
	else
	  begin
	     pulse_ctr <= pulse_ctr-1;
	     gen_ctr_l <= gen_ctr_l-1;
	     gen_ctr_r <= gen_ctr_r-1;
	  end
     end

   // Definitely *not* a 50% duty cycle; goes high immediately after
   // the data item has been latched
   assign audio_clk = pulse_ctr[12];

   assign audio_l = ~gen_ctr_l[12];
   assign audio_r = ~gen_ctr_r[12];

   
endmodule // sound
